from pathlib import Path
from rich import print
import re
import os
import fnmatch
import yaml


def load_yml_file(fpath, verbose):

    # yaml = ruamel.yaml.YAML(typ='safe')

    with open(fpath, "r") as stream:
        content = yaml.load(stream, Loader=yaml.FullLoader)
    if verbose:
        print("Loaded '%s':\n" % fpath)
        print(content)
        print("\n")

    return content


def load_yml_files(dpath, verbose):
    """
    Safe load multiple yaml files.

    All top level values are expected to be keys.
    """
    content = []
    if os.path.isdir(dpath):
        for f in get_filenames(dpath, "*.yml"):
            new_content = {
                "filename": f,
                "content": load_yml_file(os.path.join(dpath, f), verbose),
            }
            content.append(new_content)
    if verbose:
        print("Loaded '%s':\n" % dpath)
        print(content)
        print("\n")

    return content


def get_filenames(dpath, pattern):
    """Yield files in given directory matching pattern."""
    return fnmatch.filter(os.listdir(dpath), pattern)


def write_file(data, fpath):
    """write a file to disk only if content has changed."""
    Path(os.path.dirname(fpath)).mkdir(parents=True, exist_ok=True)
    if not os.path.isfile(fpath):
        with open(fpath, "w") as f:
            f.write(data)
    else:
        with open(fpath, "r+") as f:
            # replace content only if needed
            if data != f.read():
                f.truncate(0)
                f.seek(0)
                f.write(data)


def read_file(fpath):
    """
    Read a file and literaly return content.
    If file is yaml, it must skip the stream header.
    """
    with open(fpath, "r") as f:
        # skip '---' header of yaml streams
        if os.path.splitext(fpath)[1] == ".yml":
            f.readline()

        # eat up every empty lines
        pos = f.tell()
        line = f.readline()

        while not line.strip():
            pos = f.tell()
            line = f.readline()

        # go back to non-empty line
        f.seek(pos)

        return f.read()


def read_files(dpath, type, verbose):
    """
    Read every files files under a given directory.
    Return a list of dictionaries. Each dictionary contains the filename
    and the file content.
    """
    if os.path.isdir(dpath):
        dfiles = []
        for f in get_filenames(dpath, type):
            if verbose:
                print("Reading file '%s'" % os.path.join(dpath, f))
            dfiles.append({"filename": f, "content": read_file(os.path.join(dpath, f))})
        return dfiles
    else:
        if verbose:
            print("'%s' directory doesn't exist...skipping" % dpath)
        return None


def controlleveltitle(mdfiles):
    for file in mdfiles:
        if "README" not in file["filename"]:
            file["content"] = re.sub(r"((?:[#]+))", r"#\1", file["content"])
    return mdfiles

def _convert_dict_of_string(d):

    if isinstance(d, dict):
        output = {}
        for k, v in d.items():
            if isinstance(v, dict) or isinstance(v, list):
                t = yaml.dump(v)
            else:
                t = v
            output[k] = t
        return output
    else:
        return d


def convert_dict_of_string(dict):
    """ """
    output = []
    for file in dict:
        temp = {}
        temp["filename"] = file['filename']
        temp['content'] = _convert_dict_of_string(file['content'])
        output.append(temp)

    return output


def convert_string(dict):
    """ """
    output = []
    for file in dict:
        temp = {}
        temp["filename"] = file['filename']
        value =  yaml.dump(file['content'],Dumper=MyDumper)
        temp['content'] = re.sub(r'^(.)$',r'        \1', value)
        output.append(temp)

    return output


import yaml

class MyDumper(yaml.Dumper):

    def increase_indent(self, flow=False, indentless=False):
        return super(MyDumper, self).increase_indent(flow, False)


from pathlib import Path
from re import template
from ansible_gendoc import __version__
from ansible_gendoc.gendoc import Gendoc
from typing import Optional
import typer
import os
import sys
import shutil


app = typer.Typer()


def _version_callback(value: bool) -> None:
    if value:
        versionText = typer.style(
            f"ansible-gendoc v{__version__}", fg=typer.colors.RED, bg=typer.colors.WHITE
        )
        typer.echo(versionText)
        raise typer.Exit()


@app.command()
def init(
    path: Path = typer.Argument(
        default="./", help="The path of your role.", exists=True
    ),
    force: Optional[bool] = typer.Option(
        False, "--force", "-f", help="Replace existing Template README.j2 in templates folder."
    ),
):
    """
    Copy templates README.j2 from packages in templates/role folder.
    """
    role_path = os.path.join(path, "meta/main.yml")
    if not os.path.isfile(role_path):
        print("There is no role in this Path !!!!")
        raise typer.Exit(code=1)
    template_path = os.path.join(path, "templates/")
    if not os.path.isfile(os.path.join(path, "templates")):
        Path(os.path.dirname(template_path)).mkdir(parents=True, exist_ok=True)
    if not os.path.isfile(os.path.join(template_path, "README.j2")) or force:
        pkgdir = sys.modules['ansible_gendoc'].__path__[0]
        fullpath = os.path.join(pkgdir, "templates/README.j2")
        shutil.copy(fullpath, template_path)
    else:
        print("The template already exist !!!!")
        print("use --force to replace the existent.")
        raise typer.Exit(code=1)



@app.command()
def render(
    path: Path = typer.Argument(
        default="./", help="The path of your role.", exists=True
    ),
    dry_run: Optional[bool] = typer.Option(
        False,
        "--dry_run",
        "-d",
        help="dry run, Outputs pure markdown to stdout nothing is written to disk",
    ),
    verbose: Optional[bool] = typer.Option(
        False, "--verbose", "-v", help="verbose mode"
    ),
):
    """
    Build the Documentation
    """
    gendoc = Gendoc(
        dry_run=dry_run,
        verbose=verbose,
        path=path,
    )
    gendoc.render()

@app.callback()
def main(
    version: Optional[bool] = typer.Option(
        None,
        "--version",
        "-v",
        help="Show the application's version and exit.",
        callback=_version_callback,
        is_eager=True,
    ),
) -> None:
    return

if __name__ == "__main__":
    app()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ansible_gendoc']

package_data = \
{'': ['*'], 'ansible_gendoc': ['templates/*']}

install_requires = \
['GitPython>=3.1.27,<4.0.0',
 'Jinja2>=3.1.2,<4.0.0',
 'PyYAML>=6.0,<7.0',
 'giturlparse>=0.10.0,<0.11.0',
 'ruamel.yaml>=0.17.21,<0.18.0',
 'typer[all]>=0.6.1,<0.7.0']

entry_points = \
{'console_scripts': ['ansible-gendoc = ansible_gendoc.main:app']}

setup_kwargs = {
    'name': 'ansible-gendoc',
    'version': '0.1.4',
    'description': 'Ansible-gendoc build documentation of Ansible Roles.',
    'long_description': "# Ansible-Gendoc\n\n*Inspired by Felix Archambault's* [ansidoc](https://github.com/archf/ansidoc)\nproject.\n\nAn [example](example.md) generated with `ansible-gendoc`.\n\n## Features\n\n* Generate the documentation for a role located in a directory\n* Can use a personal template `README.j2` present in folder `templates`\n\n## Quickstart\n\nIf you have an existing README.md file in your role, backup it before !\n\n### Run From docker\n\nClone this project and build the image :\n\n```bash\ngit clone\nexport DOCKER_BUILDKIT=1\ndocker build . -t ansible-gendoc:0.1.0 -t ansible-gendoc:latest\ndocker run --user $(id -u):$(id -g) -it ansible_gendoc:latest help\n```\n\n### Install python package\n\nInstall the latest version `Ansible-gendoc` with `pip` or `pipx`\n\n```bash\npip install ansible-gendoc\n```\n\n### Usage\n\n```bash\nansible-gendoc --help\n\n Usage: ansible-gendoc [OPTIONS] COMMAND [ARGS]...\n\n╭─ Options ────────────────────────────────────────────────────────────────────────╮\n│ --version             -v        Show the application's version and exit.         │\n│ --install-completion            Install completion for the current shell.        │\n│ --show-completion               Show completion for the current shell, to copy   │\n│                                 it or customize the installation.                │\n│ --help                          Show this message and exit.                      │\n╰──────────────────────────────────────────────────────────────────────────────────╯\n╭─ Commands ───────────────────────────────────────────────────────────────────────╮\n│ init     Copy templates README.j2 from packages in templates/role folder.        │\n│ render   Build the Documentation                                                 │\n╰──────────────────────────────────────────────────────────────────────────────────╯\n```\n\n#### Build your first documentation of a role\n\nTo build the documentation roles, you can run these commands :\n\n* with package installed with pip\n  `ansible-gendoc render`.\n* with docker images\n  `docker run --user $(id -u):$(id -g) -v <path_role>:/role -it ansible_gendoc:latest render role`.\n\n#### Use your personal template\n\nTo use a personal template, you need to `init` the template in the templates\nfolder of your role. If `ansible-gendoc` find an existing file\n`templates/README.j2`, it will use it to render the README.md file.\n\n```bash\nansible-gendoc init\nls templates\nREADME.j2\n```\n\nThe template use [`jinja`](https://jinja.palletsprojects.com/) as templating\nlanguage.\n\nModify it, for example replace `html` or `Restructuredtext` or another language.\nYou can remove some variables too.\n\n#### Documentation of vars template\n\nThe documentation of vars coming soon.\n",
    'author': 'Stephane ROBERT',
    'author_email': 'stephane.robert@fr.clara.net',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://www.claranet.fr/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



from jinja2 import Environment, PackageLoader, FileSystemLoader
from rich import print
from pathlib import Path
from ansible_gendoc.helpers import (
    load_yml_file,
    load_yml_files,
    write_file,
    read_files,
    controlleveltitle,
    convert_dict_of_string,
    convert_string
)
from git import Repo

import subprocess
import os
import typer
from giturlparse import parse

class Gendoc:
    def __init__(self, **kwargs):
        """initiate object with provided options."""
        self.mode = kwargs.get("mode")
        self.verbose = kwargs.get("verbose")
        self.dry_run = kwargs.get("dry_run")
        self.sphinx = kwargs.get("sphinx")
        if "target" in kwargs:
            self.target = os.path.abspath(kwargs.get("target"))
        self.rolespath = os.path.abspath(kwargs.get("path"))
        self.clean = kwargs.get("clean")
        self.opts = kwargs
        self.dirpath = os.getcwd()

    def _run_command(self, command):
        """ """
        p = subprocess.Popen(
            command,
            stdout=subprocess.PIPE,
        )
        out, err = p.communicate()
        return_code = p.returncode
        return {
            "code": p.returncode,
            "out": out,
            "err": err,
        }


    def _make_role_doc(self, role):
        # Read ansible galaxy info
        if os.path.isfile(os.path.join(role["path"], "meta/.galaxy_install_info")):
            ansible_galaxy_info = load_yml_file(
                os.path.join(role["path"], "meta/.galaxy_install_info"), self.verbose
            )
        else:
            ansible_galaxy_info = None
        ## Read meta of roles
        meta_vars = load_yml_file(
            os.path.join(role["path"], "meta/main.yml"), self.verbose
        )
        # load literaly role vars/*.yml
        vars_files = load_yml_files(os.path.join(role["path"], "vars"), self.verbose)
        # load literaly role defaults/*.yml
        defaults_files = load_yml_files(
            os.path.join(role["path"], "defaults"), self.verbose
        )

        docs_md_files = read_files(os.path.join(role["path"]), "*.md", self.verbose)
        docs_md_files = controlleveltitle(docs_md_files)

        # load template and create templating environment
        if os.path.isfile(os.path.join(self.rolespath, "templates/README.j2")):
            env = Environment(
                loader=FileSystemLoader(searchpath=os.path.join(self.rolespath, "templates")),
                lstrip_blocks=True,
                trim_blocks=True,
            )
        else:
            env = Environment(
                loader=PackageLoader("ansible_gendoc", "templates"),
                lstrip_blocks=True,
                trim_blocks=True,
            )
        role_defaults_files = convert_dict_of_string(defaults_files)
        string_role_defaults_files = convert_string(defaults_files)
        # render role
        template = env.get_template("README.j2")
        # render method accepts the same arguments as the dict constructor
        t = template.render(
            self.opts,
            rolename=role["name"],
            repoowner=role["repoowner"],
            repourl=role["repourl"],
            reponame=role["reponame"],
            role_meta_vars=meta_vars,
            role_vars_files=convert_dict_of_string(vars_files),
            role_defaults_files=role_defaults_files,
            string_role_defaults_files=string_role_defaults_files,
            ansible_galaxy_info=ansible_galaxy_info,
            role_docs_md_files=docs_md_files,
        )
        if self.verbose or self.dry_run:
            print("\n*********************************************\n")
            print("Markdown Generated")
            print(t)

        if not self.dry_run:
            write_file(t, os.path.join(role["path"], "README.md"))
            if self.sphinx:
                write_file(
                    t,
                    os.path.join(
                        self.dirpath,
                        os.path.join(self.target, "source", role["target"]),
                        "readme.md",
                    ),
                )

        if self.verbose:
            print("Role '%s' ...done\n" % role["name"])

    def render(self):
        """
        Render README.md
        """
        role = {}
        if os.path.isfile(os.path.join(self.rolespath, "meta/main.yml")):
            role["name"] = os.path.basename(os.path.abspath(self.rolespath))
            role["repoowner"] = ""
            role["repourl"] = ""
            role["reponame"] = ""
            role["path"] = self.rolespath
            try:
                repo = Repo(role["path"])

                result = parse(repo.remotes.origin.url)
                role["repoowner"] = result.owner
                role["reponame"] = result.name
                role["repourl"] = result.normalized
            except:
                pass
            self._make_role_doc(role)

        else:
            print("There is no role in this Path !!!!")
            raise typer.Exit(code=1)
